----------------------------------------------------------------------------------
-- Company: Digilent RO
-- Engineer: Mircea Dabacan
-- 
-- Create Date:    15:30:29 03/02/2008 
-- Module Name:    HB5RefComp - Behavioral 
-- Project Name:   Hb5RefProj
-- Description: This is the simple Digilent HB5 Reference Component.
--              This component can also be used with PmodHB3.
--   Inputs:
--     - ck = clock signal (100MHz)
--     - bitDirIn = the direction to rotate the motor 
--     - vecDfIn = the value of the duty factor for the PWM signal.
--   Outputs:
--     - bitEnOut = Enable pin for Pmod HB5
--     - bitDirOut = Dir pin for Pmod HB5
--
--   Internal constants:
--     - intPreDiv = Pre division factor. Defines the division factor
--       used to build the internal clock (ckInt).
--       frequency(ckInt) =  frequency(ckInt)/intPreDiv.
--     - strRadix = string to define the radix used for vecDfIn and 
--       internal counter. Allowed values:
--       - "Bcd" for BCD (the whole PWM circuit works in BCD)
--       - "Hex" for hexadecimal (the whole PWM circuit works in hexadecimal)
--     - intSwDelay = number of ck periods to force bitEnOut "low" before and after
--       a state change of bitDirOut.
--     Behavioral description:
--       bitEnOut is built as a PWM signal. 
--       The frequency of bitEnOut is:
--       - frequency(ckInt)/256 - if strRadix is "Hex" 
--       - frequency(ckInt)/100 - if strRadix is "Bcd" 
--       The Duty Factor of bitEnOut is:
--       - vecDfIn/256 - if strRadix is "Hex" 
--       - vecDfIn/100 - if strRadix is "Bcd" 
--       
--       To avoid Pmod HB5 damage, bitDirOut is not allowed to change state when 
--       bitEnOut is "high" or even close to that. 
--       To make sure of this condition, when the bitDirIn change state, 
--       bitEnOut is forced "low" for intSwDelay ck periods, then bitDirOut gets
--       the value of bitDirIn, bitEnOut is forced "low" for other intSwDelay ck 
--       periods before being released to generate further PWM cycles.                          

-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
Library UNISIM;
use UNISIM.vcomponents.all;

entity HB5RefComp is
    Port ( ck : in  STD_LOGIC;          -- system clock (100MHz)
           bitDirIn : in  STD_LOGIC;    -- User direction request
           vecDfIn : in  STD_LOGIC_VECTOR(7 downto 0); -- Duty factor value 
           bitEnOut : out  STD_LOGIC:= '0';   -- Enable pin for Pmod HB5
           bitDirOut : out  STD_LOGIC:= '1'); -- Dir pin for Pmod HB5
end HB5RefComp;

architecture Behavioral of HB5RefComp is
  constant intPreDiv: integer :=2048; -- predivision factor
  -- with intPreDiv = 2048:
          -- frequency(ckInt) = 100MHz/2048 = 48.83KHz
          -- frequency(bitEnOut) = 100MHz/2048/100 = 488.3Hz (with strRadix = BCD)
          -- frequency(bitEnOut) = 100MHz/2048/256 = 190.7Hz (with strRadix = HEX)
  constant strRadix: string := "BCD"; -- used radix ("BCD" or "HEX")
  constant intSwDelay: integer := 100000; -- # of ck periods to force bitEnOut "low"
          -- before and after a change of bitDirOut state. 
			 
  signal cntPreDiv: integer range 0 to intPreDiv - 1 := 0; --counter for predivider
  signal ckInt, ckIntBuf: STD_LOGIC; -- Internal clock for PWM
  signal cntSwDelay: integer range 0 to 2*intSwDelay := 0; --counter for delay
  signal cntPwm: STD_LOGIC_VECTOR(7 downto 0) := x"00";    -- counter for PWM
  signal bitPwmInt: STD_LOGIC; -- Internal Pwm signal (before delay protection)
  signal bitDirOld: STD_LOGIC; -- old value of bitDirIn, to track changes
  
begin

   -- BUFG: Global Clock Buffer (source by an internal signal)
   --       Spartan-3E
   -- Xilinx HDL Language Template, version 13.1

   BUFG_inst : BUFG
   port map (
      O => ckIntBuf,     -- Clock buffer output
      I => ckInt      -- Clock buffer input
   );


  preDiv: process(ck)  -- clock predivision
  begin
    if ck'event and ck = '1' then
	   if cntPreDiv = intPreDiv - 1 then
		  cntPreDiv <= 0;
		  ckInt <= '1';
		else
        cntPreDiv <= cntPreDiv + 1;
		  ckInt <= '0';
      end if;
    end if;
  end process;	 

  PWM: process(ckIntBuf)  -- PWM counter
  begin
    if ckIntBuf'event and ckIntBuf = '1' then
	   if strRadix = "HEX" then  -- hexadecimal count
        cntPwm <= cntPwm + '1';
	   elsif strRadix = "BCD" then -- decimal rollover
		  if cntPwm(3 downto 0) = x"9" then
		    cntPwm(3 downto 0) <= x"0";
		    if cntPwm(7 downto 4) = x"9" then
		      cntPwm(7 downto 4) <= x"0";
          else   
		      cntPwm(7 downto 4) <= cntPwm(7 downto 4) + x"1";
          end if;
        else   
		    cntPwm(3 downto 0) <= cntPwm(3 downto 0) + x"1";
        end if;
	   else                        -- wrong value of strRadix
		  cntPwm <= x"FF";          -- cntPwm stuck at 255, as result
											 -- bitPwmInt is forced permanently "low"
      end if;
    end if;
  end process;	 

  bitPwmInt <= '1' when cntPwm < vecDfIn else  -- internal PWM signal
               '0';

  trackDir: process(ck)  -- tracking bitDirIn and generating delay
  begin
    if ck'event and ck = '1' then
      bitDirOld <= bitDirIn;               -- tracking bitDirIn
	   if bitDirOld /= bitDirIn then        -- bitDirIn changed 
		  cntSwDelay <= 0;                   -- start delay
	   elsif cntSwDelay = 2*intSwDelay then -- delay expired 
		  null;                              -- cntSwDelay stuck
		else
        cntSwDelay <= cntSwDelay + 1;      -- counting delay
      end if;
    end if;
  end process;	 

  bitEnOut <= bitPwmInt when cntSwDelay = 2*intSwDelay else -- normal operation
              '0';            -- during switching delay

  switchDirection: process(ck)  -- switching motor direction
  begin
    if ck'event and ck = '1' then
	   if cntSwDelay = intSwDelay then      -- midd delay 
		  bitDirOut <= bitDirOld;            -- switching direction
      end if;
    end if;
  end process;	 

end Behavioral;
