----------------------------------------------------------------------------------
-- Company: Digilent RO
-- Engineer: Mircea Dabacan & Mihai Corovei
-- 
-- Create Date:    15:52:39 04/20/2011 
-- Module Name:    HallTradSpeedDecoder - Behavioral 
-- Project Name:   Hb5RefProj
-- Description: 
-- 	Samples inputs SA and SB from the Hall transducers, on ck rising edge.
--    It divides System Clock in order to obtain a TimeBase for Speed Counter.
--    Based on the SA and SB sequence, increments/decrements the cntSpeed counter.
--    When TimeBase counter finished counting, cntSpeed value is saved in regSpeed
--    and cntSpeed is reset.
--    Measuring TimeBase: a 360 degree motor rotation = (Reduction Factor * 12) cntSpeed counts
--                    (6 edges detected on each SA and SB transducer signal) 
--                        TimeBase = (60 / (Reduction Factor * 12)) sec
--                               => DivFactor =  26315789 for Reduction Factor = 19.
--    Measuring Direction:  On each edge on pinSA or pinSB signal the other one is 
--       verified and according to its value cntSpeed is incremented or decremented. 
--
--    cntSpeed = 0000 if motor off
--               0000 to 0999 if motor rotates forwards
--               1999 to 1000 if motor rotates backwards
--                  (represented in BCD Two's Complement Representation Code)            
-- 
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity HallTradSpeedDecoder is
    Port ( ck : in  STD_LOGIC;
           pinSA : in  STD_LOGIC;
           pinSB : in  STD_LOGIC;
           SpeedOut : out  STD_LOGIC_VECTOR (12 downto 0)); -- output for speed and direction 
end HallTradSpeedDecoder;

architecture Behavioral of HallTradSpeedDecoder is
------------------------------------------------------------------------
-- Constant and Signal Declarations
------------------------------------------------------------------------
  
  signal regSA, regSB: std_logic := '0'; -- registered values of transducer signals
  signal oldSA, oldSB: std_logic := '0'; -- old values of transducer signals
   signal oldSA2, oldSB2: std_logic := '0'; -- second old values of transducer signals
  signal cntSpeed: STD_LOGIC_VECTOR (12 downto 0) := (others => '0'); -- Speed counter
  signal regSpeed: STD_LOGIC_VECTOR (12 downto 0) := (others => '0'); -- Speed register
  signal cntSpeedEn: std_logic:= '0';  -- save and reset cntSpeed signal
  
  constant DivFactor: integer := 26315789; -- division factor
  signal TimeBaseCounter: integer range 0 to DivFactor:= 0; -- TimeBase Counter
  
------------------------------------------------------------------------
-- Module Implementation
------------------------------------------------------------------------
  
begin
---------------------
-- Signal assignation
---------------------

-- cntSpeed Enable signal
cntSpeedEn <= '1' when TimeBaseCounter = 0 else '0';

-- convert Speed from BCD Two's Complement to Magnitude and Sign Representation Code.
SpeedOut(11 downto 0) <= regSpeed(11 downto 0) when regSpeed(12)= '0' 
	                    else x"999" - regSpeed(11 downto 0);
SpeedOut(12) <= regSpeed(12);
							  
-------------------------------
-- System Clock Devider Process
-------------------------------
TimeBaseCounterProc: process(ck)
begin
	if ck'event and ck = '1' then
		if TimeBaseCounter /= DivFactor - 1 then 
			TimeBaseCounter <= TimeBaseCounter + 1;
		else TimeBaseCounter <= 0;
		end if;
	end if;
end process;	

--------------------------
-- Measuring Speed Process
--------------------------	
SpeedCounter:process(ck, pinSA, pinSB)
begin
	if ck'event and ck = '1' then
   
		regSA <= pinSA;  -- registering the inputs
		regSB <= pinSB;
		oldSA <= regSA;  -- storing values for next iteration
		oldSB <= regSB;
		oldSA2 <= oldSA;  -- storing values for next iteration second back-up
		oldSB2 <= oldSB;
	
		if  cntSpeedEn = '1' then 
			cntSpeed <= "0000000000000";
			regSpeed <= cntSpeed;
		else 	
			if oldSA2 = '0' and oldSA = '1' and regSA = '1' then 	-- SA rising edge
				if regSB = '0' then						-- SB = 0 
					if cntSpeed(3 downto 0) /= x"9" then 
						cntSpeed(3 downto 0) <= cntSpeed(3 downto 0) + '1';-- increment speed counter
					elsif cntSpeed(7 downto 4) /= x"9" then 
						cntSpeed(7 downto 4) <= cntSpeed(7 downto 4) + '1';
						cntSpeed(3 downto 0) <= x"0";
						elsif cntSpeed(11 downto 8) /= x"9" then
							cntSpeed(11 downto 8) <= cntSpeed(11 downto 8) + '1';
							cntSpeed(7 downto 4) <= x"0";
							cntSpeed(3 downto 0) <= x"0";
							else 
								cntSpeed(12) <= '0';
								cntSpeed(11 downto 8) <= x"0";
								cntSpeed(7 downto 4) <= x"0";
								cntSpeed(3 downto 0) <= x"0";
							end if;				
				else
					if cntSpeed(3 downto 0) /= x"0" then 
						cntSpeed(3 downto 0) <= cntSpeed(3 downto 0) - '1'; -- decrement speed counter
					elsif cntSpeed(7 downto 4) /= x"0" then 
						cntSpeed(7 downto 4) <= cntSpeed(7 downto 4) - '1';
						cntSpeed(3 downto 0) <= x"9";
						elsif cntSpeed(11 downto 8) /= x"0" then
							cntSpeed(11 downto 8) <= cntSpeed(11 downto 8) - '1';
							cntSpeed(7 downto 4) <= x"9";
							cntSpeed(3 downto 0) <= x"9";
						else	
							cntSpeed(12) <= '1';
							cntSpeed(11 downto 8) <= x"9";
							cntSpeed(7 downto 4) <= x"9";
							cntSpeed(3 downto 0) <= x"9";
						end if;				-- decrement speed counter
				end if;
	  
			elsif	oldSA2 = '1' and oldSA = '0' and regSA = '0' then 	-- SA falling edge	  
				if regSB = '0' then						-- SB = 0 
					if cntSpeed(3 downto 0) /= x"0" then 
						cntSpeed(3 downto 0) <= cntSpeed(3 downto 0) - '1'; -- decrement speed counter
					elsif cntSpeed(7 downto 4) /= x"0" then 
						cntSpeed(7 downto 4) <= cntSpeed(7 downto 4) - '1';
						cntSpeed(3 downto 0) <= x"9";
						elsif cntSpeed(11 downto 8) /= x"0" then
							cntSpeed(11 downto 8) <= cntSpeed(11 downto 8) - '1';
							cntSpeed(7 downto 4) <= x"9";
							cntSpeed(3 downto 0) <= x"9";
						else	
							cntSpeed(12) <= '1';
							cntSpeed(11 downto 8) <= x"9";
							cntSpeed(7 downto 4) <= x"9";
							cntSpeed(3 downto 0) <= x"9";
						end if;				-- decrement speed counter
				else
					if cntSpeed(3 downto 0) /= x"9" then 
						cntSpeed(3 downto 0) <= cntSpeed(3 downto 0) + '1';-- increment speed counter
					elsif cntSpeed(7 downto 4) /= x"9" then 
						cntSpeed(7 downto 4) <= cntSpeed(7 downto 4) + '1';
						cntSpeed(3 downto 0) <= x"0";
						elsif cntSpeed(11 downto 8) /= x"9" then
							cntSpeed(11 downto 8) <= cntSpeed(11 downto 8) + '1';
							cntSpeed(7 downto 4) <= x"0";
							cntSpeed(3 downto 0) <= x"0";
							else 
								cntSpeed(12) <= '0';
								cntSpeed(11 downto 8) <= x"0";
								cntSpeed(7 downto 4) <= x"0";
								cntSpeed(3 downto 0) <= x"0";
							end if;				
				end if;
			end if;

			if oldSB2 = '0' and oldSB = '1' and regSB = '1' then 	-- SB rising edge
				if regSA = '1' then						-- SA = 1 
					if cntSpeed(3 downto 0) /= x"9" then 
						cntSpeed(3 downto 0) <= cntSpeed(3 downto 0) + '1';-- increment speed counter
					elsif cntSpeed(7 downto 4) /= x"9" then 
						cntSpeed(7 downto 4) <= cntSpeed(7 downto 4) + '1';
						cntSpeed(3 downto 0) <= x"0";
						elsif cntSpeed(11 downto 8) /= x"9" then
							cntSpeed(11 downto 8) <= cntSpeed(11 downto 8) + '1';
							cntSpeed(7 downto 4) <= x"0";
							cntSpeed(3 downto 0) <= x"0";
							else 
								cntSpeed(12) <= '0';
								cntSpeed(11 downto 8) <= x"0";
								cntSpeed(7 downto 4) <= x"0";
								cntSpeed(3 downto 0) <= x"0";
							end if;				
				else
					if cntSpeed(3 downto 0) /= x"0" then 
						cntSpeed(3 downto 0) <= cntSpeed(3 downto 0) - '1'; -- decrement speed counter
					elsif cntSpeed(7 downto 4) /= x"0" then 
						cntSpeed(7 downto 4) <= cntSpeed(7 downto 4) - '1';
						cntSpeed(3 downto 0) <= x"9";
						elsif cntSpeed(11 downto 8) /= x"0" then
							cntSpeed(11 downto 8) <= cntSpeed(11 downto 8) - '1';
							cntSpeed(7 downto 4) <= x"9";
							cntSpeed(3 downto 0) <= x"9";
						else	
							cntSpeed(12) <= '1';
							cntSpeed(11 downto 8) <= x"9";
							cntSpeed(7 downto 4) <= x"9";
							cntSpeed(3 downto 0) <= x"9";
						end if;				-- decrement speed counter
				end if;
			elsif	oldSB2 = '1' and oldSB = '0' and regSB = '0' then 	-- SB falling edge	  
				if regSA = '1' then						-- SA = 1 
					if cntSpeed(3 downto 0) /= x"0" then 
						cntSpeed(3 downto 0) <= cntSpeed(3 downto 0) - '1'; -- decrement speed counter
					elsif cntSpeed(7 downto 4) /= x"0" then 
						cntSpeed(7 downto 4) <= cntSpeed(7 downto 4) - '1';
						cntSpeed(3 downto 0) <= x"9";
						elsif cntSpeed(11 downto 8) /= x"0" then
							cntSpeed(11 downto 8) <= cntSpeed(11 downto 8) - '1';
							cntSpeed(7 downto 4) <= x"9";
							cntSpeed(3 downto 0) <= x"9";
						else	
							cntSpeed(12) <= '1';
							cntSpeed(11 downto 8) <= x"9";
							cntSpeed(7 downto 4) <= x"9";
							cntSpeed(3 downto 0) <= x"9";
						end if;				-- decrement speed counter
				else
					if cntSpeed(3 downto 0) /= x"9" then 
						cntSpeed(3 downto 0) <= cntSpeed(3 downto 0) + '1';-- increment speed counter
					elsif cntSpeed(7 downto 4) /= x"9" then 
						cntSpeed(7 downto 4) <= cntSpeed(7 downto 4) + '1';
						cntSpeed(3 downto 0) <= x"0";
						elsif cntSpeed(11 downto 8) /= x"9" then
							cntSpeed(11 downto 8) <= cntSpeed(11 downto 8) + '1';
							cntSpeed(7 downto 4) <= x"0";
							cntSpeed(3 downto 0) <= x"0";
							else 
								cntSpeed(12) <= '0';
								cntSpeed(11 downto 8) <= x"0";
								cntSpeed(7 downto 4) <= x"0";
								cntSpeed(3 downto 0) <= x"0";
							end if;				
				end if;
			end if;
		end if;
	end if;
end process;
end Behavioral;

